<?php
/**
 * 2020-2022 AdTela
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 * DISCLAIMER
 *
 * @author    AdTela SAS
 * @copyright 2020-2022 AdTela SAS
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *
 * @version   Release: $Revision$
 *  International Registered Trademark & Property of AdTela SAS
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

// ===============================================
class PrinterPoint extends Module
{
    // ////////
    // Constantes

    public const DS_SERVICE_URL = 'https://www.expedy.fr/api/print';
    public const DS_SERVICE_SMS_URL = 'https://www.expedy.fr/api/sms';

    public const IMPRIMANTE_MODELE_58 = 1;
    public const IMPRIMANTE_MODELE_80 = 2;

    public const AFFICHER_PRODUIT_NOM_ET_ATTRIBUTS = 1;
    public const AFFICHER_PRODUIT_REFERENCE_ET_ATTRIBUTS = 2;
    public const AFFICHER_PRODUIT_NOM_SEUL = 3;
    public const AFFICHER_PRODUIT_REFERENCE_SEULE = 4;

    public const AFFICHER_PAGE_AVEC_MONTANTS_AVEC_TVA = 1;
    public const AFFICHER_PAGE_AVEC_MONTANTS_SANS_TVA = 2;
    public const AFFICHER_PAGE_SANS_MONTANTS = 3;

    public const DECLENCHEMENT_COMMANDE = 1;
    public const DECLENCHEMENT_STATUT = 2;

    // ////////
    // Fonctions de base

    // -------------------------------------------
    // Constructeur
    public function __construct()
    {
        $this->name = 'printerpoint';
        $this->tab = 'others';
        $this->version = '1.0.3';
        $this->author = 'Expedy';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.5', 'max' => '1.8'];
        $this->bootstrap = true;
        $this->module_key = '2ab33f01186f6ce2d1e942bcd327d2c2';

        parent::__construct();

        $this->displayName = $this->l('Expedy');
        $this->description = $this->l('Print automation of your orders and dispatch docs, sms alerts ...');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall ?');
    }

    // -------------------------------------------
    // Fonction appelée lors de l'installation du module
    public function install()
    {
        if (
            !parent::install()
            || !$this->registerHook('actionOrderStatusUpdate')
            || !$this->registerHook('actionValidateOrder')
        ) {
            return false;
        }

        return true;
    }

    // -------------------------------------------
    // Fonction appelée lors de la désinstallation du module
    public function uninstall()
    {
        if (
            !parent::uninstall()
            || !Configuration::deleteByName('PRINTERPOINT_API_SID')
            || !Configuration::deleteByName('PRINTERPOINT_API_TOKEN')
            || !Configuration::deleteByName('PRINTERPOINT_API_PRINTER_ID')
            || !Configuration::deleteByName('PRINTERPOINT_API_PRINTER_ID_BACKUP')
            || !Configuration::deleteByName('PRINTERPOINT_API_SMS_NUM')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_PRINTER_MODEL')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_AUTOCUT')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_LOGO')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_ENSEIGNE')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_NOM')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_ADRESSE_LIGNE_1')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_ADRESSE_LIGNE_2')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_CODE_POSTAL')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_VILLE')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_TELEPHONE')
            || !Configuration::deleteByName('PRINTERPOINT_ENT_SIRET')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_PRODUIT')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_PAGE')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_DECLENCHEMENT')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_COPIES')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_MODE_LIVRAISON')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_MODE_PAIEMENT')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_TOTAL_AVEC_TAXES')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_TOTAL_HORS_TAXES')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_DETAIL_TAXES')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_BARCODE')
            || !Configuration::deleteByName('PRINTERPOINT_MULTISTORE')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_NB_ARTICLES')
            || !Configuration::deleteByName('PRINTERPOINT_OPT_COMMENTAIRE_CLIENT')
            || !Configuration::deleteByName('PRINTERPOINT_ERR_MESSAGES')
        ) {
            return false;
        }

        return true;
    }

    // -------------------------------------------
    // Fonction appelée pour la configuration du module
    public function getContent()
    {
        if (Tools::isSubmit('submit' . $this->name)) {
            // traitement du formulaire

            // SID
            $value = Tools::getValue('PRINTERPOINT_API_SID');
            Configuration::updateValue('PRINTERPOINT_API_SID', $value);

            // Token
            $value = Tools::getValue('PRINTERPOINT_API_TOKEN');
            Configuration::updateValue('PRINTERPOINT_API_TOKEN', $value);

            // Printer ID
            $value = Tools::getValue('PRINTERPOINT_API_PRINTER_ID');
            Configuration::updateValue('PRINTERPOINT_API_PRINTER_ID', $value);

            // Backup Printer ID
            $value = Tools::getValue('PRINTERPOINT_API_PRINTER_ID_BACKUP');
            Configuration::updateValue('PRINTERPOINT_API_PRINTER_ID_BACKUP', $value);

            // Backup Printer ID
            $value = Tools::getValue('PRINTERPOINT_API_SMS_NUM');
            Configuration::updateValue('PRINTERPOINT_API_SMS_NUM', $value);

            // Printer model
            $value = Tools::getValue('PRINTERPOINT_OPT_PRINTER_MODEL');
            Configuration::updateValue('PRINTERPOINT_OPT_PRINTER_MODEL', $value);

            // Auto Cut
            $value = Tools::getValue('PRINTERPOINT_OPT_AUTOCUT');
            Configuration::updateValue('PRINTERPOINT_OPT_AUTOCUT', $value);

            // Logo
            $value = Tools::getValue('PRINTERPOINT_ENT_LOGO');
            Configuration::updateValue('PRINTERPOINT_ENT_LOGO', $value);

            // Enseigne
            $value = Tools::getValue('PRINTERPOINT_ENT_ENSEIGNE');
            Configuration::updateValue('PRINTERPOINT_ENT_ENSEIGNE', $value);

            // Nom
            $value = Tools::getValue('PRINTERPOINT_ENT_NOM');
            Configuration::updateValue('PRINTERPOINT_ENT_NOM', $value);

            // Adresse ligne 1
            $value = Tools::getValue('PRINTERPOINT_ENT_ADRESSE_LIGNE_1');
            Configuration::updateValue('PRINTERPOINT_ENT_ADRESSE_LIGNE_1', $value);

            // Adresse ligne 2
            $value = Tools::getValue('PRINTERPOINT_ENT_ADRESSE_LIGNE_2');
            Configuration::updateValue('PRINTERPOINT_ENT_ADRESSE_LIGNE_2', $value);

            // Code postal
            $value = Tools::getValue('PRINTERPOINT_ENT_CODE_POSTAL');
            Configuration::updateValue('PRINTERPOINT_ENT_CODE_POSTAL', $value);

            // Ville
            $value = Tools::getValue('PRINTERPOINT_ENT_VILLE');
            Configuration::updateValue('PRINTERPOINT_ENT_VILLE', $value);

            // Téléphone
            $value = Tools::getValue('PRINTERPOINT_ENT_TELEPHONE');
            Configuration::updateValue('PRINTERPOINT_ENT_TELEPHONE', $value);

            // SIRET
            $value = Tools::getValue('PRINTERPOINT_ENT_SIRET');
            Configuration::updateValue('PRINTERPOINT_ENT_SIRET', $value);

            // Affichage produit
            $value = Tools::getValue('PRINTERPOINT_OPT_PRODUIT');
            Configuration::updateValue('PRINTERPOINT_OPT_PRODUIT', $value);

            // Mise en page
            $value = Tools::getValue('PRINTERPOINT_OPT_PAGE');
            Configuration::updateValue('PRINTERPOINT_OPT_PAGE', $value);

            // Déclenchement d'impression
            $value = Tools::getValue('PRINTERPOINT_OPT_DECLENCHEMENT');
            Configuration::updateValue('PRINTERPOINT_OPT_DECLENCHEMENT', $value);

            // Nombre de copies
            $value = Tools::getValue('PRINTERPOINT_OPT_COPIES');
            Configuration::updateValue('PRINTERPOINT_OPT_COPIES', $value);

            // Affichage total avec taxes
            $value = Tools::getValue('PRINTERPOINT_OPT_TOTAL_AVEC_TAXES');
            Configuration::updateValue('PRINTERPOINT_OPT_TOTAL_AVEC_TAXES', $value);

            // Affichage total sans taxes
            $value = Tools::getValue('PRINTERPOINT_OPT_TOTAL_HORS_TAXES');
            Configuration::updateValue('PRINTERPOINT_OPT_TOTAL_HORS_TAXES', $value);

            // Affichage détail des taxes
            $value = Tools::getValue('PRINTERPOINT_OPT_DETAIL_TAXES');
            Configuration::updateValue('PRINTERPOINT_OPT_DETAIL_TAXES', $value);

            // Affichage code bar d'articles
            $value = Tools::getValue('PRINTERPOINT_OPT_BARCODE');
            Configuration::updateValue('PRINTERPOINT_OPT_BARCODE', $value);

            // Multistore
            $value = Tools::getValue('PRINTERPOINT_MULTISTORE');
            Configuration::updateValue('PRINTERPOINT_MULTISTORE', $value);

            // Affichage nombre d'articles
            $value = Tools::getValue('PRINTERPOINT_OPT_NB_ARTICLES');
            Configuration::updateValue('PRINTERPOINT_OPT_NB_ARTICLES', $value);

            // Affichage commentaire du client
            $value = Tools::getValue('PRINTERPOINT_OPT_COMMENTAIRE_CLIENT');
            Configuration::updateValue('PRINTERPOINT_OPT_COMMENTAIRE_CLIENT', $value);

            // Affichage remerciement
            $value = Tools::getValue('PRINTERPOINT_OPT_REMERCIEMENT');
            Configuration::updateValue('PRINTERPOINT_OPT_REMERCIEMENT', $value);

            // Affichage mode de livraison
            $value = Tools::getValue('PRINTERPOINT_OPT_MODE_LIVRAISON');
            Configuration::updateValue('PRINTERPOINT_OPT_MODE_LIVRAISON', $value);

            // Affichage mode de paiement
            $value = Tools::getValue('PRINTERPOINT_OPT_MODE_PAIEMENT');
            Configuration::updateValue('PRINTERPOINT_OPT_MODE_PAIEMENT', $value);

            // Info after action
            $value = 'CONFIGURATION SAVED.';
            Configuration::updateValue('PRINTERPOINT_ERR_MESSAGES', $value);
        } else {
            Configuration::updateValue('PRINTERPOINT_ERR_MESSAGES', '');
        }

        return $this->displayConfiguration();
    }

    // - - - - - - - - - - - - - - - - - - - - - -
    // Afficher la page de configuration du module
    public function displayConfiguration()
    {
        $moduleName = $this->name;
        $token = Tools::getAdminTokenLite('AdminModules');
        $values = $this->getConfigurationValues();
        // $currentIndex = AdminController::$currentIndex;

        $this->context->smarty->assign(
            [
                'module_name' => $moduleName,
                'token' => $token,
                'values' => $values,
            ]
        );

        return $this->display(__FILE__, 'printerpoint_configure.tpl');
    }

    // - - - - - - - - - - - - - - - - - - - - - -
    // Récupérer les valeurs de configuration du module
    public function getConfigurationValues()
    {
        return [
            'PRINTERPOINT_API_SID' => Configuration::get('PRINTERPOINT_API_SID'),
            'PRINTERPOINT_API_TOKEN' => Configuration::get('PRINTERPOINT_API_TOKEN'),
            'PRINTERPOINT_API_PRINTER_ID' => Configuration::get('PRINTERPOINT_API_PRINTER_ID'),
            'PRINTERPOINT_API_PRINTER_ID_BACKUP' => Configuration::get('PRINTERPOINT_API_PRINTER_ID_BACKUP'),
            'PRINTERPOINT_API_SMS_NUM' => Configuration::get('PRINTERPOINT_API_SMS_NUM'),

            'PRINTERPOINT_OPT_PRINTER_MODEL' => Configuration::get('PRINTERPOINT_OPT_PRINTER_MODEL'),
            'PRINTERPOINT_OPT_AUTOCUT' => Configuration::get('PRINTERPOINT_OPT_AUTOCUT'),
            'PRINTERPOINT_ENT_LOGO' => Configuration::get('PRINTERPOINT_ENT_LOGO'),
            'PRINTERPOINT_ENT_ENSEIGNE' => Configuration::get('PRINTERPOINT_ENT_ENSEIGNE'),
            'PRINTERPOINT_ENT_NOM' => Configuration::get('PRINTERPOINT_ENT_NOM'),
            'PRINTERPOINT_ENT_ADRESSE_LIGNE_1' => Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_1'),
            'PRINTERPOINT_ENT_ADRESSE_LIGNE_2' => Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_2'),
            'PRINTERPOINT_ENT_CODE_POSTAL' => Configuration::get('PRINTERPOINT_ENT_CODE_POSTAL'),
            'PRINTERPOINT_ENT_VILLE' => Configuration::get('PRINTERPOINT_ENT_VILLE'),
            'PRINTERPOINT_ENT_TELEPHONE' => Configuration::get('PRINTERPOINT_ENT_TELEPHONE'),
            'PRINTERPOINT_ENT_SIRET' => Configuration::get('PRINTERPOINT_ENT_SIRET'),
            'PRINTERPOINT_OPT_PRODUIT' => Configuration::get('PRINTERPOINT_OPT_PRODUIT'),
            'PRINTERPOINT_OPT_PAGE' => Configuration::get('PRINTERPOINT_OPT_PAGE'),
            'PRINTERPOINT_OPT_DECLENCHEMENT' => Configuration::get('PRINTERPOINT_OPT_DECLENCHEMENT'),
            'PRINTERPOINT_OPT_COPIES' => Configuration::get('PRINTERPOINT_OPT_COPIES'),
            'PRINTERPOINT_OPT_TOTAL_AVEC_TAXES' => Configuration::get('PRINTERPOINT_OPT_TOTAL_AVEC_TAXES'),
            'PRINTERPOINT_OPT_TOTAL_HORS_TAXES' => Configuration::get('PRINTERPOINT_OPT_TOTAL_HORS_TAXES'),
            'PRINTERPOINT_OPT_DETAIL_TAXES' => Configuration::get('PRINTERPOINT_OPT_DETAIL_TAXES'),
            'PRINTERPOINT_OPT_BARCODE' => Configuration::get('PRINTERPOINT_OPT_BARCODE'),
            'PRINTERPOINT_MULTISTORE' => Configuration::get('PRINTERPOINT_MULTISTORE'),
            'PRINTERPOINT_OPT_NB_ARTICLES' => Configuration::get('PRINTERPOINT_OPT_NB_ARTICLES'),
            'PRINTERPOINT_OPT_COMMENTAIRE_CLIENT' => Configuration::get('PRINTERPOINT_OPT_COMMENTAIRE_CLIENT'),
            'PRINTERPOINT_OPT_REMERCIEMENT' => Configuration::get('PRINTERPOINT_OPT_REMERCIEMENT'),
            'PRINTERPOINT_OPT_MODE_LIVRAISON' => Configuration::get('PRINTERPOINT_OPT_MODE_LIVRAISON'),
            'PRINTERPOINT_OPT_MODE_PAIEMENT' => Configuration::get('PRINTERPOINT_OPT_MODE_PAIEMENT'),
            'PRINTERPOINT_ERR_MESSAGES' => Configuration::get('PRINTERPOINT_ERR_MESSAGES'),
        ];
    }

    // -------------------------------------------
    // Fonction appelée après la modification du statut d'une commande
    public function hookActionOrderStatusUpdate($params)
    {
        // vérifier les conditions pour imprimer le ticket :
        //  Si le module est configuré pour imprimer le ticket lors d'un changement de statut de commande
        //  Et id_order_state = 2 (Paiement accepté) ou 12 (Paiement à distance accepté)
        //  Et id_order_state = (Passage en caisse) >> Caisse Prestatill POS 3.X
        if (
            Configuration::get('PRINTERPOINT_OPT_DECLENCHEMENT') == self::DECLENCHEMENT_STATUT
            && (2 == $params['newOrderStatus']->id
                || 12 == $params['newOrderStatus']->id
                || (int) Configuration::get('PS_PRESTATILL_PAYMENT') == $params['newOrderStatus']->id)
        ) {
            $this->imprimerTicket($params['id_order']);
        }
    }

    // -------------------------------------------
    // Fonction appelée après la validation d'une commande
    public function hookActionValidateOrder($params)
    {
        // vérifier les conditions pour imprimer le ticket :
        //  Si le module est configuré pour imprimer le ticket dès la validation d'une commande
        if (self::DECLENCHEMENT_COMMANDE == Configuration::get('PRINTERPOINT_OPT_DECLENCHEMENT')) {
            $this->imprimerTicket($params['order']->id);
        }
    }

    // ////////
    // Fonctions utilitaires

    // -------------------------------------------
    // Imprime un ticket de commande
    // Si le numéro de commande est 0, un ticket de test est imprimé
    public function imprimerTicket($id_order = 0)
    {
        try {
            // récupérer les informations de la commande
            $order_infos = self::getOrderInfos($id_order);
            // construire le message
            $msg = $this->constructMessage($order_infos);
            // imprimer le message sur une imprimante réseau
            self::printWithApi($msg, $order_infos);
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    // - - - - - - - - - - - - - - - - - - - - - -
    // Récupérer les informations d'une commande d'après son numéro en vue de l'imprimer
    public static function getOrderInfos($id_order = 0)
    {
        // ////////
        // Récupérer les informations de la commande

        // start new integration
        $order_infos = [];    // tableau contenant les informations de la commande

        // call order
        $req_order = 'SELECT `id_order`,
        `id_cart`,
        `id_carrier`,
        `id_address_delivery`,
        `reference`,
        `total_paid_tax_incl`,
        `total_paid_tax_excl`,
        `date_add`,
        `payment`,
        `id_shop`,
        `id_lang` FROM `' . _DB_PREFIX_ . 'orders` WHERE `id_order` = ' . (int) $id_order . '
        ORDER BY `id_order` ASC LIMIT 1';
        $rez_order = Db::getInstance()->executeS($req_order);

        // check shipping address
        $req_shipping = 'SELECT `company`,
        `lastname`,
        `firstname`,
        `address1`,
        `address2`,
        `postcode`,
        `city`,
        `other`,
        `phone`,
        `phone_mobile` FROM `' . _DB_PREFIX_ . 'address`
        WHERE `id_address` = ' . (int) $rez_order['0']['id_address_delivery'] . '
        ORDER BY `id_address` ASC LIMIT 1';
        $rez_shipping = Db::getInstance()->executeS($req_shipping);

        // check carrier
        $req_carrier = 'SELECT `name` FROM `' . _DB_PREFIX_ . 'carrier`
        WHERE `id_carrier` = ' . (int) $rez_order['0']['id_carrier'] . '
        ORDER BY `id_carrier` ASC LIMIT 1';
        $rez_carrier = Db::getInstance()->executeS($req_carrier);

        // check message
        $req_msg = 'SELECT `message` FROM `' . _DB_PREFIX_ . 'message`
        WHERE `id_order` = ' . (int) $rez_order['0']['id_order'] . ' AND `private` = 0
        ORDER BY `id_message` ASC LIMIT 1';
        $rez_msg = Db::getInstance()->executeS($req_msg);

        if (!isset($rez_msg['0']['message'])) {
            $order_msg = '';
        } else {
            $order_msg = $rez_msg['0']['message'];
        }

        $order_infos['id_order'] = $rez_order['0']['id_order'];
        $order_infos['id_cart'] = $rez_order['0']['id_cart'];
        $order_infos['id_carrier'] = $rez_order['0']['id_carrier'];
        $order_infos['reference'] = $rez_order['0']['reference'];
        $order_infos['total_paid_tax_incl'] = $rez_order['0']['total_paid_tax_incl'];
        $order_infos['total_paid_tax_excl'] = $rez_order['0']['total_paid_tax_excl'];
        $order_infos['date_add'] = $rez_order['0']['date_add'];
        $order_infos['carrier'] = $rez_carrier['0']['name'];
        $order_infos['payment'] = $rez_order['0']['payment'];
        $order_infos['message'] = $order_msg;
        $order_infos['id_shop'] = $rez_order['0']['id_shop'];
        $order_infos['id_lang'] = $rez_order['0']['id_lang'];
        $order_infos['id_store'] = '';
        $order_infos['delivery_address'] = [
            'company' => $rez_shipping['0']['company'],
            'lastname' => $rez_shipping['0']['lastname'],
            'firstname' => $rez_shipping['0']['firstname'],
            'address1' => $rez_shipping['0']['address1'],
            'address2' => $rez_shipping['0']['address2'],
            'postcode' => $rez_shipping['0']['postcode'],
            'city' => $rez_shipping['0']['city'],
            'other' => $rez_shipping['0']['other'],
            'phone' => $rez_shipping['0']['phone'],
            'phone_mobile' => $rez_shipping['0']['phone_mobile'],
        ];

        $req_prods = 'SELECT `id_order_detail`,
        `product_id`,
        `product_reference`,
        `product_ean13`,
        `product_name`,
        `product_quantity`,
        `total_price_tax_excl`,
        `total_price_tax_incl`,
        `tax_name`,
        `product_attribute_id` FROM `' . _DB_PREFIX_ . 'order_detail`
         WHERE `id_order` = ' . (int) $rez_order['0']['id_order'] . ' ORDER BY `id_order` ASC';
        $rez_prods = Db::getInstance()->executeS($req_prods);

        foreach ($rez_prods as $rez_prod) {
            // call product
            $req_prodinfo = 'SELECT `id_category_default` FROM `' . _DB_PREFIX_ . 'product`
            WHERE `id_product` = ' . (int) $rez_prod['product_id'] . '
            ORDER BY `id_product` ASC LIMIT 1';
            $rez_prodinfo = Db::getInstance()->executeS($req_prodinfo);

            // call more product info
            $req_prodinfo_more = 'SELECT `name`,`description` FROM `' . _DB_PREFIX_ . 'product_lang`
            WHERE `id_product` = ' . (int) $rez_prod['product_id'] . '
            AND `id_shop` = ' . $order_infos['id_shop'] . '
            AND `id_lang` = ' . $order_infos['id_lang'] . '
            ORDER BY `id_product` ASC LIMIT 1';
            $rez_prodinfo_more = Db::getInstance()->executeS($req_prodinfo_more);

            // get product parent cat
            $req_prodinfo_cat = 'SELECT `id_parent` FROM `' . _DB_PREFIX_ . 'category`
            WHERE `id_category` = ' . (int) $rez_prodinfo['0']['id_category_default'] . '
            ORDER BY `id_category` ASC LIMIT 1';
            $rez_prodinfo_cat = Db::getInstance()->executeS($req_prodinfo_cat);

            // get product tax name
            $req_prodinfo_order_tax = 'SELECT `id_tax` FROM `' . _DB_PREFIX_ . 'order_detail_tax`
            WHERE `id_order_detail` = ' . (int) $rez_prod['id_order_detail'] . '
            ORDER BY `id_order_detail` ASC LIMIT 1';
            $rez_prodinfo_order_tax = Db::getInstance()->executeS($req_prodinfo_order_tax);
            $req_prodinfo_tax = 'SELECT `name`
            FROM `' . _DB_PREFIX_ . 'tax_lang`
            WHERE `id_tax` = ' . (int) $rez_prodinfo_order_tax['0']['id_tax'] . '
            AND `id_lang` = ' . $order_infos['id_lang'] . '
            ORDER BY `id_tax` ASC LIMIT 1';
            $rez_prodinfo_tax = Db::getInstance()->executeS($req_prodinfo_tax);

            // get product attributes
            $attributes_small = '';
            $attributes_array = [];
            $req_prodinfo_attributes_comb = 'SELECT `id_attribute`
            FROM `' . _DB_PREFIX_ . 'product_attribute_combination`
            WHERE `id_product_attribute` = ' . (int) $rez_prod['product_attribute_id'] . '
            ORDER BY `id_product_attribute` ASC';
            $rez_prodinfo_attributes_comb = Db::getInstance()->executeS($req_prodinfo_attributes_comb);
            $attr_num = 0;

            foreach ($rez_prodinfo_attributes_comb as $rez_prodinfo_attribute_comb) {
                $req_prodinfo_attributes = 'SELECT `name` FROM `' . _DB_PREFIX_ . 'attribute_lang`
                    WHERE `id_attribute` = ' . (int) $rez_prodinfo_attribute_comb['id_attribute'] . '
                    AND `id_lang` = ' . $order_infos['id_lang'] . '
                    ORDER BY `id_attribute` ASC LIMIT 1';
                $rez_prodinfo_attributes = Db::getInstance()->executeS($req_prodinfo_attributes);

                $attributes_array[] = $rez_prodinfo_attributes['0']['name'];

                if ($attr_num == 0) {
                    $attributes_small = $rez_prodinfo_attributes['0']['name'];
                } else {
                    $attributes_small = $attributes_small . ',' . $rez_prodinfo_attributes['0']['name'];
                }
                ++$attr_num;
            }

            // Si le module idxrcustomproduct est installé
            // Overwrites legacy attributes

            if (Module::isEnabled(Tools::strtolower('idxrcustomproduct'))) {
                // format to product attributes
                $attributes_small = '';
                $attributes_array = [];
                $clean_attributes = $rez_prodinfo_more['0']['description'];
                $clean_attributes_expl = explode('\\n', $clean_attributes);
                $findme = 'icp_code';
                $count_att = 0;
                foreach ($clean_attributes_expl as $clean_attributes_exploded) {
                    $pos = strpos($clean_attributes_exploded, $findme);

                    if ($pos === false) {
                        $attributes_array[] = $clean_attributes_exploded;
                        // $clean_attributes_small_expl = explode(': ', $clean_attributes_exploded);
                        // if ($count_att == 0) {
                        //  $attributes_small = $clean_attributes_small_expl[1];
                        // } else {
                        //  $attributes_small = $attributes_small.",".$clean_attributes_small_expl[1];
                        // }
                    }
                    ++$count_att;
                }
            }

            // End le module idxrcustomproduct est installé

            $order_infos['product_list'][] = [
                'id_order_detail' => $rez_prod['id_order_detail'],
                'product_id' => $rez_prod['product_id'],
                'reference' => $rez_prod['product_reference'],
                'product_cat_id' => $rez_prodinfo['0']['id_category_default'],
                'product_parentcat_id' => $rez_prodinfo_cat['0']['id_parent'],
                'ean13' => $rez_prod['product_ean13'],
                'name' => $rez_prodinfo_more['0']['name'],
                'cart_name' => $rez_prod['product_name'],
                'quantity' => $rez_prod['product_quantity'],
                'total' => $rez_prod['total_price_tax_excl'],
                'total_wt' => $rez_prod['total_price_tax_incl'],
                'tax_name' => $rez_prodinfo_tax['0']['name'],
                'attributes' => $attributes_array,
                'attributes_small' => $attributes_small,
            ];
        }

        // Si le module fspickupatstorecarrier est installé
        if (Module::isEnabled(Tools::strtolower('fspickupatstorecarrier'))) {
            // récupérer les informations de ce module pour cette commande
            $sql = 'SELECT `id_fspickupatstorecarrier`
                          ,`id_order`
                          ,`id_store`
                          ,`date_pickup`
                          ,`date_add`
                          ,`date_upd`
                    FROM `fspickupatstorecarrier`
                    WHERE `id_order` = ' . (int) $id_order;

            $dbResult = Db::getInstance()->executeS($sql);

            if (isset($dbResult[0])) {
                // ajouter la date de retrait en magasin aux informations de la commande
                $order_infos['pickupatstore_date_pickup'] = $dbResult[0]['date_pickup'];
            }
        }
        // End Si le module fspickupatstorecarrier est installé

        // Si le module preferreddeliverytime est installé

        if (Module::isEnabled(Tools::strtolower('preferreddeliverytime'))) {
            $id_cart_trg = $order_infos['id_cart'];

            // récupérer les informations de ce module pour cette commande

            $sql_preferred = 'SELECT `preferred_date`
                          ,`preferred_time`
                    FROM `' . _DB_PREFIX_ . 'preferred_delivery_time`
                    WHERE `id_cart` = ' . (int) $id_cart_trg;

            $dbResult_preferedtime = Db::getInstance()->executeS($sql_preferred);

            if (isset($dbResult_preferedtime[0])) {
                // ajouter la date de retrait en magasin aux informations de la commande
                $order_infos['preferred_date'] = $dbResult_preferedtime[0]['preferred_date'];
                $order_infos['preferred_time'] = $dbResult_preferedtime[0]['preferred_time'];
            }
        }

        // End Si le module preferreddeliverytime est installé

        // Si le module prestatillhomedelivery est installé
        if (Module::isEnabled(Tools::strtolower('prestatillhomedelivery'))) {
            // récupérer les informations de ce module pour cette commande
            $sql = 'SELECT `hour`
                          ,`day`
                    FROM `' . _DB_PREFIX_ . 'prestatill_homedelivery_creneau`
                    WHERE `id_cart` = ' . (int) $order_infos['id_cart'];

            $ptillhomedelivery_dbx = Db::getInstance()->executeS($sql);

            if (isset($ptillhomedelivery_dbx[0])) {
                // ajouter la date de retrait en magasin aux informations de la commande
                $order_infos['ptillhdelivery_date_pkup'] = date('d-m-Y', strtotime($ptillhomedelivery_dbx[0]['day']))
                    . ' - ' . $ptillhomedelivery_dbx[0]['hour'];
                // get carrier "name"
                $sql_next = 'SELECT `name`
                       FROM `' . _DB_PREFIX_ . 'carrier`
                       WHERE `id_carrier` = ' . (int) $order_infos['id_carrier'];
                $prestatillhomedelivery_db_nxt = Db::getInstance()->executeS($sql_next);
                if (isset($prestatillhomedelivery_db_nxt[0])) {
                    $order_infos['prestatillhomedelivery_carrier_name'] = $prestatillhomedelivery_db_nxt[0]['name'];
                }
                // end get carrier "name"
            }
        }
        // End Si le module prestatillhomedelivery est installé

        // Si le module prestatilldrive est installé

        if (Module::isEnabled(Tools::strtolower('prestatilldrive'))) {
            // récupérer les informations de ce module pour cette commande
            $sql = 'SELECT `hour`
                          ,`day`
                          ,`id_store`
                    FROM `' . _DB_PREFIX_ . 'prestatill_drive_creneau`
                    WHERE `id_cart` = ' . (int) $order_infos['id_cart'];

            $prestatilldrive_dbx = Db::getInstance()->executeS($sql);

            if (isset($prestatilldrive_dbx[0])) {
                // ajouter la date de retrait en magasin aux informations de la commande
                $order_infos['prestatilldrive_date_pickup'] = date('d-m-Y', strtotime($prestatilldrive_dbx[0]['day']))
                    . ' - ' . $prestatilldrive_dbx[0]['hour'];

                // get carrier "name"
                $sql_next = 'SELECT `name`
                                    ,`address1`
                       FROM `' . _DB_PREFIX_ . 'store_lang`
                       WHERE `id_store` = ' . (int) $prestatilldrive_dbx[0]['id_store'];
                $prestatilldrive_dbResult_next = Db::getInstance()->executeS($sql_next);
                if (isset($prestatilldrive_dbResult_next[0])) {
                    $order_infos['prestatilldrive_carrier_name'] = $prestatilldrive_dbResult_next[0]['name']
                        . ' ' . $prestatilldrive_dbResult_next[0]['address1'];

                    $order_infos['id_store'] = $prestatilldrive_dbx[0]['id_store'];
                }
                // end get carrier "name"
            }
        }

        // End Si le module prestatilldrive est installé

        // Si le module prestadrive est installé
        if (Module::isEnabled(Tools::strtolower('prestadrive'))) {
            // récupérer les informations de ce module pour cette commande
            $sql = 'SELECT `complet_drive_time`
                          ,`hour_drive_time`
                    FROM `' . _DB_PREFIX_ . 'prestadrive_order`
                    WHERE `id_prestashop_cart` = ' . (int) $order_infos['id_cart'];

            $dbResultx = Db::getInstance()->executeS($sql);

            if (isset($dbResultx[0])) {
                // ajouter la date de retrait en magasin aux informations de la commande
                $order_infos['prestadrive_date_pickup'] = date('d-m-Y', strtotime($dbResultx[0]['complet_drive_time']))
                    . '' . $dbResultx[0]['hour_drive_time'];

                // get carrier "name"
                $sql_next = 'SELECT `name`
                       FROM `' . _DB_PREFIX_ . 'carrier`
                       WHERE `id_carrier` = ' . (int) $order_infos['id_carrier'];
                $dbResult_next = Db::getInstance()->executeS($sql_next);
                if (isset($dbResult_next[0])) {
                    $order_infos['prestadrive_carrier_name'] = $dbResult_next[0]['name'];
                }
                // end get carrier "name"
            }
        }
        // End Si le module prestadrive est installé

        // retourner le tableau contenant les informations de la commande
        return $order_infos;
    }

    // - - - - - - - - - - - - - - - - - - - - - -
    // Construire le message à imprimer d'après les informations de la commande et les paramètres du module
    public function constructMessage($order_infos)
    {
        // Pour éviter de tester l'existance d'une information à chacune de ses utilisations
        //  on mes une chaîne vide par défaut.
        if (!isset($order_infos['reference'])) {
            $order_infos['reference'] = '';
        }
        if (!isset($order_infos['delivery_address'])) {
            $order_infos['delivery_address'] = [];
        }
        if (!isset($order_infos['delivery_address']['lastname'])) {
            $order_infos['delivery_address']['lastname'] = '';
        }
        if (!isset($order_infos['delivery_address']['firstname'])) {
            $order_infos['delivery_address']['firstname'] = '';
        }
        if (!isset($order_infos['delivery_address']['address1'])) {
            $order_infos['delivery_address']['address1'] = '';
        }
        if (!isset($order_infos['delivery_address']['address2'])) {
            $order_infos['delivery_address']['address2'] = '';
        }
        if (!isset($order_infos['delivery_address']['postcode'])) {
            $order_infos['delivery_address']['postcode'] = '';
        }
        if (!isset($order_infos['delivery_address']['city'])) {
            $order_infos['delivery_address']['city'] = '';
        }
        if (!isset($order_infos['delivery_address']['phone'])) {
            $order_infos['delivery_address']['phone'] = '';
        }
        if (!isset($order_infos['delivery_address']['phone_mobile'])) {
            $order_infos['delivery_address']['phone_mobile'] = '';
        }

        // déterminer la largeur maximum suivant le modèle d'imprimante
        if (self::IMPRIMANTE_MODELE_80 == Configuration::get('PRINTERPOINT_OPT_PRINTER_MODEL')) {
            $lineWidth = 48; // imprimante 80mm
            $xpdy_printer_size = 80;
        } else {
            $lineWidth = 30; // imprimante 58mm
            $xpdy_printer_size = 58;
        }

        // Logo (les noms de fichiers étant sensibles à la casse, il ne faut pas mettre l'URL en majuscules)

        $msgLogo = '';

        if ('' != Configuration::get('PRINTERPOINT_ENT_LOGO')) {
            $msgLogo .= '<IMG>' . Configuration::get('PRINTERPOINT_ENT_LOGO') . '</IMG>';
        }

        // Entête

        $msgEntete = '';

        if ('' != Configuration::get('PRINTERPOINT_ENT_ENSEIGNE')) {
            $msgEntete .= '<L><CB>' . Configuration::get('PRINTERPOINT_ENT_ENSEIGNE') . "</CB></L>\n";
            $msgEntete = self::stripAccents($msgEntete); // supprimer les accents
            $msgEntete = mb_strtoupper($msgEntete);      // mettre en majuscules
        }

        // Adresse du magasin

        $msgAdresseMagasin = '';

        if ('' != Configuration::get('PRINTERPOINT_ENT_NOM')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_NOM') . "\n";
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_1')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_1') . "\n";
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_2')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_ADRESSE_LIGNE_2') . "\n";
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_CODE_POSTAL')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_CODE_POSTAL');

            if ('' != Configuration::get('PRINTERPOINT_ENT_VILLE')) {
                $msgAdresseMagasin .= ' ';
            }
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_VILLE')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_VILLE');
        }

        if (
            '' != Configuration::get('PRINTERPOINT_ENT_CODE_POSTAL')
            || '' != Configuration::get('PRINTERPOINT_ENT_VILLE')
        ) {
            $msgAdresseMagasin .= "\n\n";
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_TELEPHONE')) {
            $msgAdresseMagasin .= Configuration::get('PRINTERPOINT_ENT_TELEPHONE') . "\n";
        }

        if ('' != Configuration::get('PRINTERPOINT_ENT_SIRET')) {
            $msgAdresseMagasin .= "\n" . Configuration::get('PRINTERPOINT_ENT_SIRET') . "\n";
        }

        if ('' != $msgAdresseMagasin) {
            $msgAdresseMagasin = '<C>' . $msgAdresseMagasin . '</C>';
            $msgAdresseMagasin = self::stripAccents($msgAdresseMagasin); // supprimer les accents
            $msgAdresseMagasin = mb_strtoupper($msgAdresseMagasin);      // mettre en majuscules
        }

        // Référence & date de la commande

        $msgCommandeReferenceDate = '';

        if ('' != $order_infos['reference']) {
            $msgCommandeReferenceDate .= '<L>Commande ' . $order_infos['reference'] . '</L>';
        }

        if (isset($order_infos['date_add'])) {
            //            $msgCommandeReferenceDate .= "<C>".date('d/m/Y H:i', strtotime($order_infos['date_add']))."</C>\n";
            $msgCommandeReferenceDate .= date('d/m/Y H:i', strtotime($order_infos['date_add'])) . "\n";
        }

        if (isset($order_infos['preferred_date'])) {
            $msgCommandeReferenceDate .= 'RETRAIT PREVU LE : ';
            $msgCommandeReferenceDate .= $order_infos['preferred_date'] . "\nENTRE " . $order_infos['preferred_time'] . "\n";
        }

        if ('' != $msgCommandeReferenceDate) {
            $msgCommandeReferenceDate = self::stripAccents($msgCommandeReferenceDate); // supprimer les accents
            $msgCommandeReferenceDate = mb_strtoupper($msgCommandeReferenceDate);      // mettre en majuscules
        }

        // Adresse de livraison

        $msgAdresseLivraison = '';

        if (isset($order_infos['delivery_address'])) {
            if (
                '' != $order_infos['delivery_address']['lastname']
                || '' != $order_infos['delivery_address']['firstname']
            ) {
                $msgAdresseLivraison .= '<BOLD><L>';
            }

            if ('' != $order_infos['delivery_address']['lastname']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['lastname'];

                if ('' != $order_infos['delivery_address']['firstname']) {
                    $msgAdresseLivraison .= ' ';
                }
            }

            if ('' != $order_infos['delivery_address']['firstname']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['firstname'];
            }

            if (
                '' != $order_infos['delivery_address']['lastname']
                || '' != $order_infos['delivery_address']['firstname']
            ) {
                $msgAdresseLivraison .= "</L></BOLD>\n";
            }

            if ('' != $order_infos['delivery_address']['address1']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['address1'] . "\n";
            }

            if ('' != $order_infos['delivery_address']['address2']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['address2'] . "\n";
            }

            if ('' != $order_infos['delivery_address']['postcode']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['postcode'];

                if ('' != $order_infos['delivery_address']['city']) {
                    $msgAdresseLivraison .= ' ';
                }
            }

            if ('' != $order_infos['delivery_address']['city']) {
                $msgAdresseLivraison .= $order_infos['delivery_address']['city'];
            }

            if (
                '' != $order_infos['delivery_address']['postcode']
                || '' != $order_infos['delivery_address']['city']
            ) {
                $msgAdresseLivraison .= "\n";
            }

            if ('' != $order_infos['delivery_address']['phone']) {
                $msgAdresseLivraison .= '<L>' . $order_infos['delivery_address']['phone'] . "</L>\n";
            }

            if ('' != $order_infos['delivery_address']['phone_mobile']) {
                $msgAdresseLivraison .= '<L>' . $order_infos['delivery_address']['phone_mobile'] . "</L>\n";
            }
        }

        if ('' != $msgAdresseLivraison) {
            $msgAdresseLivraison = '<C>' . $msgAdresseLivraison . '</C>';
            $msgAdresseLivraison = self::stripAccents($msgAdresseLivraison); // supprimer les accents
            $msgAdresseLivraison = mb_strtoupper($msgAdresseLivraison);      // mettre en majuscules
        }

        // Commande

        $msgCommande = '';

        $taxes = [];
        if (isset($order_infos['product_list'])) {
            foreach ($order_infos['product_list'] as $value) {
                // quantité (nom | référence) {prix}
                //   {attributs}

                // start get configurator product details

                $product_id_targ = $value['product_id'];
                $ean13 = $value['ean13'];

                $id_order_detail_targ = $value['id_order_detail'];
                $order_id_targ = $order_infos['id_order'];
                // start get configurator product details
                $prod_data_echo = '';

                if (Module::isEnabled(Tools::strtolower('idxrcustomproduct'))) {
                    $prod_data_echo = '';
                    // $prod_data_echo = $value['attributes_small'];
                    $count_atrs = 0;
                    foreach ($value['attributes'] as $prod_data_val) {
                        if ($count_atrs == 0) {
                            $prod_data_echo = $prod_data_echo . "\n" . $prod_data_val;
                        } else {
                            $prod_data_echo = $prod_data_echo . "\n" . $prod_data_val;
                        }
                        ++$count_atrs;
                    }

                    if ($prod_data_echo != '') {
                        $prod_data_echo = $prod_data_echo . "\n";
                    }
                }

                if (Module::isEnabled(Tools::strtolower('configurator'))) {
                    // récupérer les informations de ce module pour cette commande

                    $sql_prod_conf_detl_trg = 'SELECT `detail`
                        FROM `' . _DB_PREFIX_ . 'configurator_cart_detail`
                        WHERE `id_order` = ' . (int) $order_id_targ . '
                        AND `id_product` = ' . (int) $product_id_targ . '
                        AND `id_order_detail` = ' . (int) $id_order_detail_targ;

                    $dbResult_prod_conf_detl_trg = Db::getInstance()->executeS($sql_prod_conf_detl_trg);

                    if (isset($dbResult_prod_conf_detl_trg[0])) {
                        // ajouter la date de retrait en magasin aux informations de la commande

                        $json_prod_data_trg = $dbResult_prod_conf_detl_trg[0]['detail'];

                        // Loop through configurator data and parse for echo
                        $someObject_trg = '';
                        $someObject_trg = json_decode($json_prod_data_trg);

                        foreach ($someObject_trg as $value_trg) {
                            $prod_data_echo = $prod_data_echo . "\n" . $value_trg->name . ':';
                            $someObject_trg_step1 = $value_trg->options;
                            $trg_count = 0;
                            foreach ($someObject_trg_step1 as $value_trg_s1) {
                                if ($value_trg_s1->selected == 1) {
                                    if ($trg_count == 0) {
                                        $prod_data_echo = $prod_data_echo . ' ' . $value_trg_s1->name;
                                    } else {
                                        $prod_data_echo = $prod_data_echo . ',' . $value_trg_s1->name;
                                    }
                                    ++$trg_count;
                                }
                            }
                            $prod_data_echo = $prod_data_echo . '';
                        }
                    }
                }
                // end get configurator product details

                // Si le module an_productfields est install

                if (Module::isEnabled(Tools::strtolower('an_productfields'))) {
                    $prod_data_echo = '';
                    $prod_data_echo = Tools::substr($value['cart_name'], Tools::strlen($value['name']));

                    $prod_data_echo = preg_replace('/-[\s\S]+?:/', '', $prod_data_echo);
                    $prod_data_echo = str_replace(';', ',', $prod_data_echo);
                    $prod_data_echo = str_replace("\xc2\xa0", '', $prod_data_echo);
                    $prod_data_echo = str_replace('€', 'e', $prod_data_echo);

                    $prod_data_echo = str_replace('   ', '', $prod_data_echo);

                    if ($prod_data_echo != '') {
                        $prod_data_echo = "\n" . $prod_data_echo;
                    }
                }
                // end get an_productfields product details

                // quantité
                $product_qty = (mb_strlen($value['quantity']) == 1 ? ' ' : '') . $value['quantity'] . ' ';
                $product_qty = $value['quantity'] . ' ';

                // {prix}
                if (self::AFFICHER_PAGE_AVEC_MONTANTS_AVEC_TVA == Configuration::get('PRINTERPOINT_OPT_PAGE')) {
                    $product_price = ' ' . number_format($value['total_wt'], 2);
                } elseif (self::AFFICHER_PAGE_AVEC_MONTANTS_SANS_TVA == Configuration::get('PRINTERPOINT_OPT_PAGE')) {
                    $product_price = ' ' . number_format($value['total'], 2);
                } else {
                    $product_price = '';
                }

                // (nom | référence)
                if (
                    self::AFFICHER_PRODUIT_NOM_ET_ATTRIBUTS == Configuration::get('PRINTERPOINT_OPT_PRODUIT')
                    || self::AFFICHER_PRODUIT_NOM_SEUL == Configuration::get('PRINTERPOINT_OPT_PRODUIT')
                ) {
                    $product_name = $value['name'];         // afficher le nom du produit
                } else {
                    $product_name = $value['reference'];    // afficher la référence du produit
                }

                if ('on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_BARCODE'))) {
                    if ($ean13 != '') {
                        $ean13_data = '<EAN>' . $ean13 . '</EAN>
';
                    } else {
                        $ean13_data = '';
                    }
                }

                // $space_left = $lineWidth - mb_strlen($product_qty) - mb_strlen($product_price);
                // if ($space_left > mb_strlen($product_name)) {
                //     $product_name .= str_repeat(" ", $space_left - mb_strlen($product_name));
                // } else {
                //     $product_name = mb_substr($product_name, 0, $space_left);
                //  }

                // removed for configurator detailed prod mode
                // $msgCommande .= $product_qty.$product_name.$product_price."\n";
                // removed for configurator detailed prod mode

                $product_name = self::stripAccents($product_name);

                $actual_left_txt_space = $lineWidth - mb_strlen($product_qty . 'X ' . $product_name . $product_price . 'e');

                if ($actual_left_txt_space > 0) {
                    $add_some_spaces = str_repeat(' ', $actual_left_txt_space);
                } else {
                    $add_some_spaces = '';
                }

                $msgCommande .= $product_qty . 'X ' . $product_name . $add_some_spaces;
                $msgCommande .= $product_price . 'e' . $prod_data_echo . "\n" . $ean13_data;

                // {attributs}
                if (
                    '' != $value['attributes_small']
                    && (self::AFFICHER_PRODUIT_NOM_ET_ATTRIBUTS == Configuration::get('PRINTERPOINT_OPT_PRODUIT')
                        || self::AFFICHER_PRODUIT_REFERENCE_ET_ATTRIBUTS == Configuration::get('PRINTERPOINT_OPT_PRODUIT'))
                ) {
                    $msgCommande .= '- ' . mb_substr($value['attributes_small'], 0, $lineWidth - 6) . "\n";
                }

                if (isset($taxes[$value['tax_name']])) {
                    $taxes[$value['tax_name']] += ($value['total_wt'] - $value['total']);
                } else {
                    $taxes[$value['tax_name']] = ($value['total_wt'] - $value['total']);
                }
            }
        }

        if ('' != $msgCommande) {
            if ($xpdy_printer_size == 58) {
                $data_printer_payload = '--------------------------------
#  |       Produit      |  Prix 
--------------------------------
';
            } elseif ($xpdy_printer_size == 80) {
                $data_printer_payload = '------------------------------------------------
#  |              Produit              |  Prix  
------------------------------------------------
';
            } else {
                $data_printer_payload = '--------------------------------
#  |       Produit      |  Prix 
--------------------------------
';
            }

            $msgCommande = $data_printer_payload . $msgCommande;
            $msgCommande = self::stripAccents($msgCommande); // supprimer les accents
            // $msgCommande = mb_strtoupper($msgCommande);
            // mettre en majuscules

            if ($xpdy_printer_size == 58) {
                $data_printer_payload = '________________________________
';
            } elseif ($xpdy_printer_size == 80) {
                $data_printer_payload = '________________________________________________
';
            } else {
                $data_printer_payload = '________________________________
';
            }

            $msgCommande = $msgCommande . $data_printer_payload;
        }

        // Total à payer (TTC)

        $msgTotalTTC = '';

        if (
            'on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_TOTAL_AVEC_TAXES'))
            && isset($order_infos['total_paid_tax_incl'])
        ) {
            $msgTotalTTC .= $this->l('incl tax') . ': ' . number_format($order_infos['total_paid_tax_incl'], 2) . "\n";
            $msgTotalTTC = self::stripAccents($msgTotalTTC); // supprimer les accents
            $msgTotalTTC = mb_strtoupper($msgTotalTTC);      // mettre en majuscules
        }

        // Total à payer (HT)

        $msgTotalHT = '';

        if (
            'on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_TOTAL_HORS_TAXES'))
            && isset($order_infos['total_paid_tax_excl'])
        ) {
            $msgTotalHT .= $this->l('excl tax') . ': ' . number_format($order_infos['total_paid_tax_excl'], 2) . "\n";
            $msgTotalHT = self::stripAccents($msgTotalHT); // supprimer les accents
            $msgTotalHT = mb_strtoupper($msgTotalHT);      // mettre en majuscules
        }

        // Détail des taxes

        $msgTaxesDetail = '';

        if ('on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_DETAIL_TAXES'))) {
            foreach ($taxes as $key => $value) {
                if ($key != '') {
                    $msgTaxesDetail .= $key . ': ' . number_format($value, 2) . "\n";
                }
            }

            $msgTaxesDetail = self::stripAccents($msgTaxesDetail); // supprimer les accents
            $msgTaxesDetail = mb_strtoupper($msgTaxesDetail);      // mettre en majuscules
        }

        // Nombre d'articles

        $msgArticlesNb = '';

        if (
            'on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_NB_ARTICLES'))
            && isset($order_infos['product_list'])
        ) {
            $msgArticlesNb .= $this->l('Nb art') . ': ' . count($order_infos['product_list']) . "\n";
            $msgArticlesNb = self::stripAccents($msgArticlesNb); // supprimer les accents
            $msgArticlesNb = mb_strtoupper($msgArticlesNb);      // mettre en majuscules
        }

        // Message de remerciement

        $msgRemerciements = '';

        if ('on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_REMERCIEMENT'))) {
            $msgRemerciements .= $this->l('Thank you for your order.') . "\n";
            $msgRemerciements = self::stripAccents($msgRemerciements); // supprimer les accents
            $msgRemerciements = mb_strtoupper($msgRemerciements);      // mettre en majuscules
        }

        // Mode de livraison

        $msgModeLivraison = '';
        $isdatemsgresult = 0;

        if ('on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_MODE_LIVRAISON'))) {
            // Si le module fspickupatstorecarrier est installé
            if (
                Module::isEnabled(Tools::strtolower('fspickupatstorecarrier'))
                && isset($order_infos['pickupatstore_date_pickup'])
            ) {
                $dateMsgModeLivraison = Tools::displayDate($order_infos['pickupatstore_date_pickup'], null, true);
                $msgModeLivraison .= $this->l('Pickup date') . ' ' . $dateMsgModeLivraison . "\n";
                $isdatemsgresult = 1;
            }
            if (
                Module::isEnabled(Tools::strtolower('prestatillhomedelivery'))
                && $order_infos['ptillhdelivery_date_pkup'] != ''
            ) {
                $dateMsgModeLivraison = $order_infos['prestatillhomedelivery_carrier_name']
                    . ' ' . $order_infos['ptillhdelivery_date_pkup'];
                $msgCommandeReferenceDate .= '' . $dateMsgModeLivraison . "\n";
                $isdatemsgresult = 1;
            }
            if (
                Module::isEnabled(Tools::strtolower('prestatilldrive'))
                && $order_infos['prestatilldrive_date_pickup'] != ''
            ) {
                $dateMsgModeLivraison = $order_infos['prestatilldrive_carrier_name']
                    . "\n" . $order_infos['prestatilldrive_date_pickup'];
                $msgCommandeReferenceDate .= '' . $dateMsgModeLivraison . "\n";
                $isdatemsgresult = 1;
            }
            if (Module::isEnabled(Tools::strtolower('prestadrive')) && $order_infos['prestadrive_date_pickup'] != '') {
                $dateMsgModeLivraison = $order_infos['prestadrive_carrier_name']
                    . "\n" . $order_infos['prestadrive_date_pickup'];
                $msgCommandeReferenceDate .= '' . $dateMsgModeLivraison . "\n";
                $isdatemsgresult = 1;
            }
            if (isset($order_infos['carrier']) && $isdatemsgresult == 0) {
                $msgModeLivraison .= $order_infos['carrier'] . "\n";
            } else {
            }

            $msgModeLivraison = $msgModeLivraison . "\n";
            $msgModeLivraison = self::stripAccents($msgModeLivraison); // supprimer les accents
            $msgModeLivraison = mb_strtoupper($msgModeLivraison);      // mettre en majuscules
        }

        // Mode de paiement

        $msgModePaiement = '';

        if (
            'on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_MODE_PAIEMENT'))
            && isset($order_infos['payment'])
        ) {
            $msgModePaiement .= 'Moyen de paiement : ' . $order_infos['payment'] . "\n";
            $msgModePaiement = self::stripAccents($msgModePaiement); // supprimer les accents
            // $msgModePaiement = mb_strtoupper($msgModePaiement);      // mettre en majuscules
        }

        // Commentaire du client

        $msgMessage = '';

        if (
            'on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_COMMENTAIRE_CLIENT'))
            && isset($order_infos['message'])
        ) {
            $msgMessage .= html_entity_decode($order_infos['message']);
            $msgMessage = self::stripAccents($msgMessage); // supprimer les accents
            //  $msgMessage = mb_strtoupper($msgMessage);      // mettre en majuscules
        }

        // Assembler les différentes parties du message.

        $msg = '';

        $msg .= $msgLogo;
        $msg .= $msgEntete;

        if ('' != $msgCommandeReferenceDate) {
            $msg .= $msgCommandeReferenceDate;
        }
        $msg .= $msgModePaiement;
        $msg .= $msgModeLivraison;

        if ('' != $msgAdresseLivraison) {
            $msg .= $msgAdresseLivraison;
        }

        if ('' != $msgCommande) {
            $msg .= $msgCommande;
        }

        if ('' != $msgArticlesNb) {
            $msg .= "\n" . $msgArticlesNb;
        }

        $msg .= $msgTotalTTC;
        $msg .= $msgTotalHT;
        $msg .= $msgTaxesDetail;

        if ('' != $msgMessage) {
            $msg .= "\n<C><BOLD><L>NOTE CLIENT</L></BOLD></C>\n" . $msgMessage;
        }

        if ('' != $msgAdresseMagasin) {
            $msg .= "\n" . $msgAdresseMagasin;
        }
        // if ('' != $msgRemerciements)         $msg .= "\n".$msgRemerciements;

        // découpage du ticket
        if ('on' == Tools::strtolower(Configuration::get('PRINTERPOINT_OPT_AUTOCUT'))) {
            $msg .= '<CUT/>';
        }

        // retourner le message
        return $msg;
    }

    // - - - - - - - - - - - - - - - - - - - - - - - -
    // Enlever les accents d'un texte
    public static function stripAccents($str)
    {
        $str = htmlentities($str, ENT_NOQUOTES, 'utf-8');

        $str = preg_replace('#&([A-za-z])(?:acute|cedil|caron|circ|grave|orn|ring|slash|th|tilde|uml);#', '\1', $str);
        $str = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $str);    // pour les ligatures e.g. '&oelig;'

        $str = html_entity_decode($str, ENT_NOQUOTES, 'utf-8');

        return $str;
    }

    // - - - - - - - - - - - - - - - - - - - - - - - -
    // Imprimer sur une imprimante à distance
    public static function printWithApi($msg, $order_infos)
    {
        $this_host = $_SERVER['HTTP_HOST'];
        $order_id_final = $order_infos['id_order'];
        $order_total_final = $order_infos['total_paid_tax_incl'];
        $id_store = $order_infos['id_store'];

        $SID = Configuration::get('PRINTERPOINT_API_SID');
        $TOKEN = Configuration::get('PRINTERPOINT_API_TOKEN');

        $printer_uid = Configuration::get('PRINTERPOINT_API_PRINTER_ID');

        $printer_uid_backup = Configuration::get('PRINTERPOINT_API_PRINTER_ID_BACKUP');

        $xpdy_sms_num = Configuration::get('PRINTERPOINT_API_SMS_NUM');

        $printer_multistore = Configuration::get('PRINTERPOINT_MULTISTORE');
        // $printer_multistore = "6:DESNBXP887K||7:DESHKTI4EOW||8:PTDETIOXZQ||";

        if ($id_store != '') {
            $search_targ = $id_store;
            $search_found = '';
            $data_string_array = explode('||', $printer_multistore);
            $pp_nbx = 0;
            foreach ($data_string_array as $pp_value) {
                $data_string_subarray = explode(':', $pp_value);

                if ($search_targ == $data_string_subarray[0]) {
                    $search_found = $data_string_subarray[1];
                    break;
                }
                ++$pp_nbx;
            }
            if ($search_found != '') {
                $printer_uid = $search_found;
            }
        }

        $nb_copies = Configuration::get('PRINTERPOINT_OPT_COPIES');

        if ($printer_uid != '') {
            $printer_msg = $msg;

            $data_params = [
                'printer_uid' => $printer_uid,
                'printer_msg' => $printer_msg,
            ];

            $data_request = [
                'sid' => $SID,
                'token' => $TOKEN,
                'origin' => $this_host,
                'params' => $data_params,
            ];

            // créer et configurer une requête vers l'api

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, self::DS_SERVICE_URL);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data_request));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

            // exécuter la requête pour chaque copie demandée
            for ($i = 0; $i < $nb_copies; ++$i) {
                $result = curl_exec($ch);
            }

            curl_close($ch);
        }

        if ($printer_uid_backup != '') {
            $printer_msg = $msg;

            $data_params_backup = [
                'printer_uid' => $printer_uid_backup,
                'printer_msg' => $printer_msg,
            ];

            $data_request_backup = [
                'sid' => $SID,
                'token' => $TOKEN,
                'origin' => $this_host,
                'params' => $data_params_backup,
            ];

            // créer et configurer une requête vers l'api

            $ch_backup = curl_init();
            curl_setopt($ch_backup, CURLOPT_URL, self::DS_SERVICE_URL);
            curl_setopt($ch_backup, CURLOPT_POST, true);
            curl_setopt($ch_backup, CURLOPT_POSTFIELDS, http_build_query($data_request_backup));
            curl_setopt($ch_backup, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch_backup, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch_backup, CURLOPT_SSL_VERIFYPEER, false);

            // exécuter la requête pour chaque copie demandée
            for ($i_backup = 0; $i_backup < $nb_copies; ++$i_backup) {
                curl_exec($ch_backup);
            }
            curl_close($ch_backup);
        }

        if ($xpdy_sms_num != '') {
            $sms_sender = 'EXPEDY';

            if ($this_host != '') {
                $sms_msg = 'Nouvelle commande : ' . $order_id_final
                    . ' (' . number_format($order_total_final, 2, '.', '') . 'EUR) 
sur ' . $this_host . ' ';
            } else {
                $sms_msg = 'Nouvelle commande : ' . $order_id_final
                    . ' (' . number_format($order_total_final, 2, '.', '') . 'EUR) ';
            }
            // end params

            $data_params_sms = [
                'sms_sender' => $sms_sender,
                'sms_phone' => $xpdy_sms_num,
                'sms_msg' => $sms_msg,
            ];

            // construction requete :

            $data_request_sms = [
                'sid' => $SID,
                'token' => $TOKEN,
                'origin' => $this_host,
                'params' => $data_params_sms,
            ];

            // créer et configurer une requête vers l'api

            $ch_sms = curl_init();
            curl_setopt($ch_sms, CURLOPT_URL, self::DS_SERVICE_SMS_URL);
            curl_setopt($ch_sms, CURLOPT_POST, true);
            curl_setopt($ch_sms, CURLOPT_POSTFIELDS, http_build_query($data_request_sms));
            curl_setopt($ch_sms, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch_sms, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch_sms, CURLOPT_SSL_VERIFYPEER, false);

            // exécuter la requête pour chaque copie demandée

            // curl_exec($ch_sms);

            curl_close($ch_sms);
        }

        return $result;
    }
}
// FIN class PrinterPoint
